function [textBox1] = chan_Enh_MSER(rgb_Image,gray_Image)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Channel Enhanced MSER Text Detection
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

warning off

colorImage = rgb_Image;
I = gray_Image;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Detect MSER regions for all 3 channels and also for Grayscale.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[mserRegions1, mserConnComp1] = detectMSERFeatures(colorImage(:,:,1),'RegionAreaRange',[40 50000],'ThresholdDelta',3);
[mserRegions2, mserConnComp2] = detectMSERFeatures(colorImage(:,:,2),'RegionAreaRange',[40 50000],'ThresholdDelta',3);
[mserRegions3, mserConnComp3] = detectMSERFeatures(colorImage(:,:,3),'RegionAreaRange',[40 50000],'ThresholdDelta',3);
[mserRegions4, mserConnComp4] = detectMSERFeatures(I(:,:,:),'RegionAreaRange',[40 50000],'ThresholdDelta',3);
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Geometric filtring for all the detected regions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 1:4
    if i==1
        mserConnComp = mserConnComp1;
        mserRegions = mserRegions1;
    elseif i ==2
        mserConnComp = mserConnComp2;
        mserRegions = mserRegions2;
    elseif i ==3
        mserConnComp = mserConnComp3;
        mserRegions = mserRegions3;
    else
        mserConnComp = mserConnComp4;
        mserRegions = mserRegions4;
    end

    
% Use regionprops to measure MSER properties
mserStats = regionprops(mserConnComp, 'BoundingBox', 'Eccentricity',...
    'Solidity', 'Extent', 'Euler', 'Image', 'Area');


% Compute the aspect ratio using bounding box data.
bbox = vertcat(mserStats.BoundingBox);
w = bbox(:,3);
h = bbox(:,4);
aspectRatio = w./h;


% Threshold the data to determine which regions to remove. These thresholds
% may need to be tuned for other images.
filterIdx = aspectRatio' > 3;
filterIdx = filterIdx | [mserStats.Eccentricity] > .995 ;
filterIdx = filterIdx | [mserStats.Solidity] < 0.25;
filterIdx = filterIdx | [mserStats.Extent] < 0.03 | [mserStats.Extent] > 1;
filterIdx = filterIdx | [mserStats.EulerNumber] < -4;
filterIdx = filterIdx | [mserStats.Area] < 100;


% Remove regions
mserStats(filterIdx) = [];
mserRegions(filterIdx) = [];

% Get bounding boxes for all the regions
bboxes = vertcat(mserStats.BoundingBox);

% Convert from the [x y width height] bounding box format to the [xmin ymin
% xmax ymax] format for convenience.
xmin = bboxes(:,1);
ymin = bboxes(:,2);
xmax = xmin + bboxes(:,3) - 1;
ymax = ymin + bboxes(:,4) - 1;

% Expand the bounding boxes by a small amount.
expansionAmount = 0.02;
xmin = (1-expansionAmount) * xmin;
ymin = (1-expansionAmount) * ymin;
xmax = (1+expansionAmount) * xmax;
ymax = (1+expansionAmount) * ymax;

% Clip the bounding boxes to be within the image bounds
xmin = max(xmin, 1);
ymin = max(ymin, 1);
xmax = min(xmax, size(I,2));
ymax = min(ymax, size(I,1));

% Show the expanded bounding boxes
expandedBBoxes = [xmin ymin xmax-xmin+1 ymax-ymin+1];

% Compute the overlap ratio
overlapRatio = bboxOverlapRatio(expandedBBoxes, expandedBBoxes, 'min');

% Set the overlap ratio between a bounding box and itself to zero to
% simplify the graph representation.

n = size(overlapRatio,1); 
overlapRatio(1:n+1:n^2) = 0;

% made the weigths zero if they are less than 0.85
indices = find(overlapRatio < 0.78);
overlapRatio(indices) = 0;

% Create the graph
g = graph(overlapRatio);

% Find the connected text regions within the graph
componentIndices = conncomp(g);


% Merge the boxes based on the minimum and maximum dimensions.
xmin = accumarray(componentIndices', xmin, [], @min);
ymin = accumarray(componentIndices', ymin, [], @min);
xmax = accumarray(componentIndices', xmax, [], @max);
ymax = accumarray(componentIndices', ymax, [], @max);

% Compose the merged bounding boxes using the [x y width height] format.
textBBoxes = [xmin ymin xmax-xmin+1 ymax-ymin+1];

if i==1
    textBBoxes1 = textBBoxes;
elseif i==2
    textBBoxes2 = textBBoxes;
elseif i==3
    textBBoxes3 = textBBoxes;
else 
    textBBoxes4 = textBBoxes;
end

end

textBox = [textBBoxes1; textBBoxes2; textBBoxes3; textBBoxes4];

xmin1 = textBox(:,1);
ymin1 = textBox(:,2);
xmax1 = xmin1 + textBox(:,3) - 1;
ymax1 = ymin1 + textBox(:,4) - 1;

% Show the expanded bounding boxes
expandedBBoxes1 = [xmin1 ymin1 xmax1-xmin1+1 ymax1-ymin1+1];

% Compute the overlap ratio
overlapRatio1 = bboxOverlapRatio(expandedBBoxes1, expandedBBoxes1, 'min');

% Set the overlap ratio between a bounding box and itself to zero to
% simplify the graph representation.

n = size(overlapRatio1,1); 
overlapRatio1(1:n+1:n^2) = 0;

% made the weigths zero if they are less than 0.85
indices = find(overlapRatio1 < 0.78);
overlapRatio1(indices) = 0;

% Create the graph
g1 = graph(overlapRatio1);

% Find the connected text regions within the graph
componentIndices1 = conncomp(g1);


% Merge the boxes based on the minimum and maximum dimensions.
xmin1 = accumarray(componentIndices1', xmin1, [], @min);
ymin1 = accumarray(componentIndices1', ymin1, [], @min);
xmax1 = accumarray(componentIndices1', xmax1, [], @max);
ymax1 = accumarray(componentIndices1', ymax1, [], @max);

% Compose the merged bounding boxes using the [x y width height] format.
textBox1 = [xmin1 ymin1 xmax1-xmin1+1 ymax1-ymin1+1];

end