%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MSER Based Text Detection
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

warning off

colorImage = imread('3.jpg');
I = rgb2gray(colorImage);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Detect MSER regions.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[mserRegions1, mserConnComp1] = detectMSERFeatures(colorImage(:,:,1),'RegionAreaRange',[40 50000],'ThresholdDelta',3);
[mserRegions2, mserConnComp2] = detectMSERFeatures(colorImage(:,:,2),'RegionAreaRange',[40 50000],'ThresholdDelta',3);
[mserRegions3, mserConnComp3] = detectMSERFeatures(colorImage(:,:,3),'RegionAreaRange',[40 50000],'ThresholdDelta',3);
% [mserRegions2, mserConnComp2] = detectMSERFeatures(I,'RegionAreaRange',[100 4000],'ThresholdDelta',4);
% [mserRegions3, mserConnComp3] = detectMSERFeatures(I,'RegionAreaRange',[400 50000],'ThresholdDelta',4);
% 
% 
% figure
% imshow(I)
% hold on
% 
% plot(mserRegions, 'showPixelList', true,'showEllipses',false)
% title(' MSER Regions')
% hold off

for i = 1:3
    if i==1
        mserConnComp = mserConnComp1;
        mserRegions =mserRegions1;
    elseif i ==2
        mserConnComp = mserConnComp2;
        mserRegions = mserRegions2;
    else
        mserConnComp = mserConnComp3;
        mserRegions =mserRegions3;
    end
% Use regionprops to measure MSER properties
mserStats = regionprops(mserConnComp, 'BoundingBox', 'Eccentricity',...
    'Solidity', 'Extent', 'Euler', 'Image', 'Area');

% Compute the aspect ratio using bounding box data.
bbox = vertcat(mserStats.BoundingBox);
w = bbox(:,3);
h = bbox(:,4);
aspectRatio = w./h;


% Threshold the data to determine which regions to remove. These thresholds
% may need to be tuned for other images.
filterIdx = aspectRatio' > 3;
filterIdx = filterIdx | [mserStats.Eccentricity] > .995 ;
filterIdx = filterIdx | [mserStats.Solidity] < 0.25;
filterIdx = filterIdx | [mserStats.Extent] < 0.03 | [mserStats.Extent] > 1;
filterIdx = filterIdx | [mserStats.EulerNumber] < -4;
filterIdx = filterIdx | [mserStats.Area] < 100;


% Remove regions
mserStats(filterIdx) = [];
mserRegions(filterIdx) = [];

% 
% figure
% imshow(I)
% hold on
% 
% plot(mserRegions, 'showPixelList', true,'showEllipses',false)
% title('After Removing Non-Text Regions Based On Geometric Properties')
% hold off
% 
% 
% figure ;
% imshow(I); hold on;
% 
% for k = 1 : length(mserStats)
%   thisBB = mserStats(k).BoundingBox;
%   rectangle('Position', [thisBB(1),thisBB(2),thisBB(3),thisBB(4)],...
%   'EdgeColor','r','LineWidth',2 )
% end 

% Get bounding boxes for all the regions
bboxes = vertcat(mserStats.BoundingBox);

% Convert from the [x y width height] bounding box format to the [xmin ymin
% xmax ymax] format for convenience.
xmin = bboxes(:,1);
ymin = bboxes(:,2);
xmax = xmin + bboxes(:,3) - 1;
ymax = ymin + bboxes(:,4) - 1;

% Expand the bounding boxes by a small amount.
expansionAmount = 0.02;
xmin = (1-expansionAmount) * xmin;
ymin = (1-expansionAmount) * ymin;
xmax = (1+expansionAmount) * xmax;
ymax = (1+expansionAmount) * ymax;

% Clip the bounding boxes to be within the image bounds
xmin = max(xmin, 1);
ymin = max(ymin, 1);
xmax = min(xmax, size(I,2));
ymax = min(ymax, size(I,1));

% Show the expanded bounding boxes
expandedBBoxes = [xmin ymin xmax-xmin+1 ymax-ymin+1];

% Compute the overlap ratio
overlapRatio = bboxOverlapRatio(expandedBBoxes, expandedBBoxes, 'min');

% Set the overlap ratio between a bounding box and itself to zero to
% simplify the graph representation.

n = size(overlapRatio,1); 
overlapRatio(1:n+1:n^2) = 0;

% made the weigths zero if they are less than 0.85
indices = find(overlapRatio<0.78);
overlapRatio(indices) = 0;

% Create the graph
g = graph(overlapRatio);

% Find the connected text regions within the graph
componentIndices = conncomp(g);


% Merge the boxes based on the minimum and maximum dimensions.
xmin = accumarray(componentIndices', xmin, [], @min);
ymin = accumarray(componentIndices', ymin, [], @min);
xmax = accumarray(componentIndices', xmax, [], @max);
ymax = accumarray(componentIndices', ymax, [], @max);

% Compose the merged bounding boxes using the [x y width height] format.
textBBoxes = [xmin ymin xmax-xmin+1 ymax-ymin+1];
if i==1
    textBBoxes1 = textBBoxes;
elseif i==2
    textBBoxes2 = textBBoxes;
else
    textBBoxes3 = textBBoxes;
end
end


% Remove bounding boxes that only contain one text region
% numRegionsInGroup = histcounts(componentIndices);
% textBBoxes(numRegionsInGroup == 1, :) = [];

% Show the final text detection result.
% ITextRegion = insertShape(colorImage, 'Rectangle', textBBoxes,'LineWidth',3);

figure;
imshow(colorImage); hold on;
title('Detected Text');
% 
[r,c] = size(textBBoxes1);
for k = 1 : r
  thisBB = textBBoxes1(k,:);
  rectangle('Position', [thisBB(1),thisBB(2),thisBB(3),thisBB(4)],...
  'EdgeColor','r','LineWidth',2 );
end

% figure;
% imshow(colorImage); hold on;
% title('Detected Text');
% 
[r1,c1] = size(textBBoxes2);
for k = 1 : r1
  thisBB = textBBoxes2(k,:);
  rectangle('Position', [thisBB(1),thisBB(2),thisBB(3),thisBB(4)],...
  'EdgeColor','r','LineWidth',2 );
end

% figure;
% imshow(colorImage); hold on;
% title('Detected Text');
% % 
[r2,c2] = size(textBBoxes3);
for k = 1 : r2
  thisBB = textBBoxes3(k,:);
  rectangle('Position', [thisBB(1),thisBB(2),thisBB(3),thisBB(4)],...
  'EdgeColor','r','LineWidth',2 );
end

